#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <regex.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>

extern int	errno;
#if !defined(linux) && !defined(__NetBSD__)
extern char	*sys_errlist [];
#endif


int		match(string, pattern)
char *string;
char *pattern;
{
	int status;

#if defined(__sgi) || defined(sun)
	if (re_comp(pattern) != 0) {
#else
	regex_t re;

	if (regcomp(&re, pattern, REG_EXTENDED|REG_NOSUB) != 0) {
#endif
		return(0);     /* report error */
	}
#if defined(__sgi) || defined(sun)
	status = re_exec(string);
	if (status != 1) {
#else
	status = regexec(&re, string, (size_t) 0, NULL, 0);
	regfree(&re);
	if (status != 0) {
#endif
		return(0);     /* report error */
	}
	return(1);
}



void	usage (progname)
char	*progname;
{
	fprintf (stderr, "usage: %s [-vu=] [-f <file>] [var-name] [var-name ...]\n",
		progname);
	fprintf (stderr, "or:    %s -r [u=] [-f <file>] [reg. pattern] [reg. pattern ...]\n",
		progname);
	fputs ("       r=reg. expressions | v=without var-name | u=allow any var-names\n", stderr);
	fputs ("       = more thane one equal-sign\n", stderr);
	exit (1);
}



int		main (argc, argv)
int		argc;
char	**argv;
{
	char	line [1024];
	int		c;
	char	*cp, *var, *value, *progname;
	int		found = 0, errflg = 0;
	char	*infilename = NULL;
	FILE	*infile = stdin;
	int		i;
	int		reg = 0, novar = 0, equal = 0, unspec = 0;

	extern char *optarg;
	extern int  optind;

	progname = *argv;
	if ((argc > 1) && (!strcmp (*argv, "-h") || !strcmp (*argv, "-?")))
		usage (progname);

	while ((c = getopt (argc, argv, "f:ruv=")) != EOF)
		switch (c) {
			case 'f':
				infilename = optarg;
				break;
			case 'r':
				reg = 1;
				break;
			case 'u':
				unspec = 1;
				break;
			case 'v':
				novar = 1;
				break;
			case '=':
				equal = 1;
				break;
			case '?':
				errflg++;
				break;
		}
	if (errflg)
		usage (progname);

	argc -= optind;
	argv += optind;

	if (!argc)
		usage (progname);
	if ((infilename != NULL) && (infile = fopen (infilename, "r")) == NULL) {
		fprintf (stderr, "%s: can't open file %s (%s)\n",
			progname, infilename, sys_errlist [errno]);
		exit (1);
	}

	while (fgets (line,sizeof(line),infile) != NULL) {
		if (equal)
			value = strchr (line, '=');
		else
			value = strrchr (line, '=');
		if (value != NULL){
			/* keine whitespaces um value */
			*value++='\0';
			for (value; isspace (*value); value++);
			cp = value + strlen (value) - 1;
			while (isspace (*cp))
				--cp;
			*(cp+1) = '\0';
			if (*value == '"' && *cp == '"') {
				++value;
				*cp-- = '\0';
			}
			else if (*value == '\'' && *cp == '\'') {
				++value;
				*cp-- = '\0';
			}

			/* keine quotes */
			if (strchr (value, '"') || strchr (value, '\'') || strchr (value, '`'))
				continue;

			/* keine whitespaces um var */
			for (var = line; isspace (*var); var++);
			if (*var == '=') {
				continue;
			}

			if (unspec) {
				cp = var + strlen (var) - 1;
				while (isspace (*cp))
					--cp;
				*(cp+1) = '\0';
			}
			else {
				for (cp = var; isalnum (*cp) || *cp == '_'; cp++);
				*cp = '\0';
			}

			if (argc) {
				for (i = 0; i < argc; i++) {
					if ((*var && reg && match (var, *(argv+i))) || !strcmp (var, *(argv+i))) {
						if (novar)
							puts (value);
						else if (unspec)
							printf ("\"%s\"=\"%s\"\n", var, value);
						else
							printf ("%s=\"%s\"\n", var, value);
						found++;
					}
				}
			}
			else {
				if ((*var && reg && match (var, *argv)) || !strcmp (var, *argv)) {
					if (novar)
						puts (value);
					else
						printf ("%s=\"%s\"\n", var, value);
					found++;
				}
			}
		}
	}
	if (!found || (!reg && found != argc))
		exit (1);
}
